Package: common
Type: Package
Title: Solutions for Common Problems in Base R
Version: 1.1.3
Authors@R: c(
    person(given = "David",
           family = "Bosak",
           role = c("aut", "cre"),
           email = "dbosak01@gmail.com"), 
    person(given = "Duong",
           family = "Tran",
           role = c("ctb"),
           email = "trand000@aol.com")
           )
Maintainer: David Bosak <dbosak01@gmail.com>
Description: Contains functions for solving commonly encountered problems while
    programming in R. This package is intended to provide a lightweight 
    supplement to Base R, and will be useful for almost any R user.
License: CC0
Encoding: UTF-8
URL: https://common.r-sassy.org, https://github.com/dbosak01/common
BugReports: https://github.com/dbosak01/common/issues
Depends: R (>= 3.6.0)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), glue, box, rstudioapi
Enhances: base
Imports: utils
Config/testthat/edition: 3
RoxygenNote: 7.3.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-04-05 09:25:09 UTC; dbosa
Author: David Bosak [aut, cre],
  Duong Tran [ctb]
Repository: CRAN
Date/Publication: 2024-04-05 15:23:06 UTC
Built: R 4.4.0; ; 2024-04-28 18:09:36 UTC; unix
