# Generated by roxygen2: do not edit by hand

S3method(labels,data.frame)
S3method(sort,data.frame)
export("%eq%")
export("%p%")
export("labels<-")
export(Sys.path)
export(changed)
export(copy.attributes)
export(dir.find)
export(file.find)
export(find.names)
export(roundup)
export(source.all)
export(spaces)
export(subsc)
export(supsc)
export(symbol)
export(v)
import(utils)
